

warning off;

//%------------------------------------------------------------
//% Declare endogenous and exogenous variables
//%------------------------------------------------------------


var
aj
ak
arr
az
b    
bnot
c    
c1
data_ctot
data_dp
data_dwtot
data_ik   
data_q
data_r
data_rnot   
dp   
dp1
dp2
dw
dw1
h1
k
lm
maxlev
n    
qk
r    
rnot
uc   
uc1  
uh
uh1
w    
w1   ;

varexo 
eps_j
eps_k
eps_p
eps_r
eps_w
eps_z ;


//%-----------------------------------------------------------------------
//% Declare model parameters
//%------------------------------------------------------------------------

parameters BETA BETA1 EC EH ETA JEI M ALPHA PHIK DK LAGP LAGW PIBAR
SIGMA TAYLOR_P TAYLOR_R TAYLOR_Y TETAP TETAW XP_SS XW_SS 
RHO_J RHO_K RHO_P RHO_R RHO_W RHO_Z
STD_J STD_K STD_P STD_R STD_W STD_Z RHOD;






//%------------------------------------------------------------
//% Model equations
//%------------------------------------------------------------

model;

# llr = 1 / BETA ;
# llrk = llr - (1-DK) ;
# llxp = XP_SS ;
# llxw = XW_SS ;
# llxw1 = XW_SS ;
# lllm = (1 - BETA1/BETA) / (1 - BETA1*RHOD/PIBAR) ;
# QHTOC = JEI/(1-BETA);
# QH1TOC1 = JEI/(1-BETA1-lllm*M*(1-RHOD));
# KTOY = ALPHA/(llxp*llrk);
# BTOQH1 = M*(1-RHOD)/(1-RHOD/PIBAR) ;
# C1TOY = (1-ALPHA)*SIGMA/(1+(1/BETA-1)*BTOQH1*QH1TOC1)*(1/llxp) ;
# CTOY = (1-C1TOY-DK*KTOY) ;
# lln = ((1-SIGMA)*(1-ALPHA)/(llxp*llxw*CTOY))^(1/(1+ETA));
# lln1 = (SIGMA*(1-ALPHA)/(llxp*llxw1*C1TOY))^(1/(1+ETA));
# lly = KTOY^(ALPHA/(1-ALPHA))*(lln^(1-SIGMA))*lln1^SIGMA ;
# llctot = lly-DK*KTOY*lly;
# llik = DK*KTOY*lly; 
# llk = KTOY*lly; 
# llq = QHTOC*CTOY*lly + QH1TOC1*C1TOY*lly ;


//% 2
uc = BETA * r/dp(1)*uc(1);

//% 4
((1+data_q)*llq)*uc = uh + BETA*((1+data_q(+1))*llq)*uc(1);

//% 5
c1 + ((1+data_q)*llq)*(h1-h1(-1)) + r(-1)*b(-1)/dp = SIGMA*w*n/(1-SIGMA) + b ;

//% 6
uc1*(1-lm) = BETA1 * (r/dp(1)-RHOD*lm(+1)/dp(1))*uc1(1)  ;

//% 8
((1+data_q)*llq)*uc1 = uh1 + BETA1*((1+data_q(+1))*llq)*uc1(1) + lm*(1-RHOD)*uc1*M*((1+data_q)*llq) ;

//% 9
(c + c1 + (1+data_ik)*llik ) = n^((1-ALPHA)*(1-SIGMA))*(SIGMA*w*n/((1-SIGMA)*w1))^((1-ALPHA)*SIGMA)*k(-1)^ALPHA ;

//% 12
log(dp/PIBAR) - LAGP*log(dp(-1)/PIBAR) = BETA*(log(dp(1)/PIBAR)-LAGP*log(dp/PIBAR)) -
((1-TETAP)*(1-BETA*TETAP)/TETAP)*log(((1-ALPHA)*(1-SIGMA)*(c + c1 + (1+data_ik)*llik)/(w*n))/XP_SS) + eps_p  ;

//% 13
log(dw/PIBAR) - LAGW*log(dw(-1)/PIBAR) = BETA*(log(dw(+1)/PIBAR)-LAGW*log(dw/PIBAR)) - 
((1-TETAW)*(1-BETA*TETAW)/TETAW)*log(w*uc/(az*n^ETA)/XW_SS) + eps_w ;

//% 14
log(dw1/PIBAR) - LAGW*log(dw1(-1)/PIBAR) = BETA*(log(dw1(+1)/PIBAR)-LAGW*log(dw1/PIBAR)) - 
((1-TETAW)*(1-BETA*TETAW)/TETAW)*log((w1*uc1/(az*((SIGMA)*w*n/((1-SIGMA)*w1))^ETA))/XW_SS) + eps_w ;

//% 15
log(rnot) = TAYLOR_R*log(r(-1)) 
+ (1-TAYLOR_R)*(TAYLOR_P)*(0.25*log(dp/PIBAR)+0.25*log(dp1/PIBAR)+0.25*log(dp2/PIBAR)+0.25*log(dp2(-1)/PIBAR)) 
+ (1-TAYLOR_R)*TAYLOR_Y*(log((c + c1 + (1+data_ik)*llik)/lly)) + (1-TAYLOR_R)*log(PIBAR/BETA) + log(arr) ;
dp1=dp(-1);
dp2=dp1(-1);



//% 16-21
uc = (1-EC)/(1-BETA*EC)*(az/(c-EC*c(-1))-BETA*EC*az(+1)/(c(+1)-EC*c)) ;
uc1 = (1-EC)/(1-BETA1*EC)*(az/(c1-EC*c1(-1))-BETA1*EC*az(+1)/(c1(+1)-EC*c1)) ;
uh = (1-EH)/(1-BETA*EH)*JEI*(az*aj/(1-h1-EH*(1-h1(-1)))-BETA*EH*az(+1)*aj(+1)/((1-h1(+1))-EH*(1-h1))) ;
uh1 = (1-EH)/(1-BETA1*EH)*JEI*(az*aj/(h1-EH*h1(-1))-BETA1*EH*az(+1)*aj(+1)/(h1(+1)-EH*h1)) ;

//% 23-26
uc*qk*(1-PHIK*(data_ik-data_ik(-1))) = 
uc - BETA*uc(+1)*qk(+1)*PHIK*(data_ik(+1)-data_ik)   ;

uc*qk/ak =
BETA*uc(+1)*(ALPHA*w(+1)*n(+1)/((1-ALPHA)*(1-SIGMA)*k)+qk(+1)*(1-DK)/ak(+1))   ;

k/ak = (1+data_ik)*llik + (1-DK)*k(-1)/ak ;


//% 27-28
dw = w*dp/w(-1);
dw1 = w1*dp/w1(-1);


//% SHOCKS

log(aj) = RHO_J*log(aj(-1)) + eps_j ;
log(ak) = RHO_K*log(ak(-1)) + eps_k ;
log(arr) = RHO_R*log(arr(-1)) + eps_r ;
log(az) = RHO_Z*log(az(-1)) + eps_z ;


//% observation equations
data_ctot = (c + c1)/llctot-1 ;

data_r = r-PIBAR/BETA;   
data_rnot = rnot-PIBAR/BETA;   
data_dp = log(dp/PIBAR);
data_dwtot =log((SIGMA*dw1 + (1-SIGMA)*dw)/PIBAR);
   

//% OBC 10
lm=0;
bnot = (1-RHOD)*M*((1+data_q)*llq)*h1+RHOD*b(-1)/dp ;
maxlev = b-bnot;
log(r) = log(rnot) ;


end;



